<?php

class MBLConnectorAdmin
{
    public function __construct()
    {
        $this->_setHooks();
    }

    private function _setHooks()
    {
        if(is_admin()) {
            $this->_addConfig();
            $this->_addScripts();
        }
    }

    private function _addConfig()
    {
        add_action('mbl_options_items_after', [$this, 'addConfigOption'], 85);
        add_action('mbl_options_content_after', [$this, 'addConfigContent'], 85);
        add_filter('mbl_connector_allow_woocommerce', '__return_true');
    }

    public function addConfigOption()
    {
        echo MBLConnectorView::getPartial('option_item', 'admin');
    }

    public function addConfigContent()
    {
        echo MBLConnectorView::getPartial('options_content', 'admin', ['plugins' => $this->_getPluginsList(), 'theme' => $this->_getThemeAssets()]);
    }

    private function _addScripts()
    {
        add_action('admin_enqueue_scripts', [$this, 'addAdminStyles']);
        add_action('admin_enqueue_scripts', [$this, 'addAdminScripts']);
    }

    public function addAdminStyles()
    {
        wp_enqueue_style('mbl_connector_settings_style', plugins_url('/mbl-connector/assets/css/settings.css'), [], MBL_CONNECTOR_VERSION);
    }

    public function addAdminScripts()
    {
        wp_enqueue_script('mbl_connector_settings_script', plugins_url('/mbl-connector/assets/js/admin.js'), [], MBL_CONNECTOR_VERSION, true);
    }

    private function _getPluginsList()
    {
        $allPlugins = get_plugins();
        $plugins = [];

        foreach ($allPlugins as $path => $plugin) {
            if (!is_plugin_active($path)) {
                continue;
            }

            $alias = wpm_array_get(explode('.', wpm_array_get(explode('/', $path), 0, '')), 0);

            if (MBLConnectorCore::isDefaultPluginAsset($plugin['Name']) || MBLConnectorCore::isDefaultPluginAsset($alias)) {
                continue;
            }

            $plugins[$alias] = $plugin['Name'];
        }

        return apply_filters('mbl_connector_admin_plugins', $plugins);
    }

    private function _getThemeAssets()
    {
        $themePathArray = explode('/', get_template_directory_uri());
        $alias = end($themePathArray);

        return apply_filters('mbl_connector_admin_themes', [
            'name'  => wp_get_theme()->name,
            'alias' => $alias,
        ]);
    }
}