<?php

class MBLConnectorCore
{
    const THEME_SLUG = 'mbl-connector-theme';
    const PRIORITY_MIN = -999999999;
    const PRIORITY_MAX = 999999999;

    private static $_defaultOptions = [
        'mbl_connector' => [
            'plugins' => [],
            'themes'  => [],
        ],
    ];

    public function __construct()
    {
        if ($this->_pluginEnabled()) {
            $this->_run();
        } elseif (is_admin()) {
            add_action('admin_notices', [$this, 'notActiveNotice']);
        }
    }

    public function notActiveNotice()
    {
        $class = 'notice notice-error';
        $message = __('КОННЕКТОР не активирован!', 'mbl_admin');

        if (!defined('WP_MEMBERSHIP_VERSION')) {
            $message .= ' ' . __('Не установлен плагин MEMBERLUX.', 'mbl_admin');

        } elseif (version_compare(WP_MEMBERSHIP_VERSION, '2.9.9.9.9.9', '<')) {
            $message .= ' ' . __('Устаревшая версия MEMBERLUX.', 'mbl_admin');

        } elseif (!$this->_pluginKeyActive()) {
            $message .= ' ' . __('Ключ не активирован.', 'mbl_admin');
        }

        printf('<div class="%1$s"><p>%2$s</p></div>', esc_attr($class), esc_html($message));
    }

    private function _pluginEnabled()
    {
        return defined('WP_MEMBERSHIP_VERSION')
               && !version_compare(WP_MEMBERSHIP_VERSION, '2.9.9.9.9.9', '<')
               && $this->_pluginKeyActive();
    }

    private function _pluginKeyActive()
    {
        $activeKeys = wpm_array_pluck(wpm_array_get(wpm_get_key_data(), 'keys.active', array()), 'type');

        if (count(wpm_array_filter($activeKeys, 'fullpackaccess'))) {
            return true;
        }

        return (bool)count(wpm_array_filter($activeKeys, 'connector'));
    }

    private function _run()
    {
        new MBLConnectorAdmin();
        new MBLConnectorPublic();
    }

    public static function install()
    {
        defined('WP_MEMBERSHIP_VERSION') || exit;

        self::_setDefaultOptions();
        self::_installConnectorTheme();
    }

    private static function _setDefaultOptions()
    {
        foreach (self::$_defaultOptions as $key => $value) {
            self::_setDefaultOption($key, $value);
        }
    }

    private static function _setDefaultOption($key, $value)
    {
        if (is_array($value)) {
            foreach ($value as $_key => $_value) {
                self::_setDefaultOption($key . '.' . $_key, $_value);
            }
        } elseif (wpm_get_option($key) === null) {
            self::setOption($key, $value);
        }
    }

    public static function setOption($key, $value)
    {
        $main_options = get_option('wpm_main_options');

        if (!isset($main_options) || !is_array($main_options)) {
            $main_options = [];
        }

        update_option('wpm_main_options', wpm_array_set($main_options, $key, $value));
    }

    private static function _installConnectorTheme()
    {
        $path = get_theme_root();
        $themePath = $path . '/' . self::THEME_SLUG;

        if (!is_dir($themePath)) {
            mkdir($themePath, 0755);
            file_put_contents($themePath . '/style.css', '');
            file_put_contents($themePath . '/style.min.css', '');
        }
    }

    private static function _uninstallConnectorTheme()
    {
        $path = get_theme_root();
        $themePath = $path . '/' . self::THEME_SLUG;

        if (file_exists($themePath . '/style.min.css')) {
            unlink($themePath . '/style.min.css');
        }
        if (file_exists($themePath . '/style.css')) {
            unlink($themePath . '/style.css');
        }
        if (is_dir($themePath)) {
            unlink($themePath);
        }
    }

    public static function uninstall()
    {
        self::_uninstallConnectorTheme();
    }

    public static function isDefaultPluginAsset($handle)
    {
        return strpos(strtolower($handle), 'memberlux') !== false
               || (strtolower($handle) == 'woocommerce' && apply_filters('mbl_connector_allow_woocommerce', function_exists('is_checkout') && is_checkout()))
               || strpos(strtolower($handle), 'member-luxe') !== false
               || strpos(strtolower($handle), 'mbl-') !== false
               || strpos(strtolower($handle), 'wppage') !== false;
    }

    public static function isDisabledAsset($handle)
    {
        $disabled = [
            'jquery',
        ];

        return (strtolower($handle) == 'woocommerce' && !apply_filters('mbl_connector_allow_woocommerce', function_exists('is_checkout') && is_checkout()))
               || in_array(strtolower($handle), $disabled);
    }
}
