<?php

class MBLConnectorMonitor
{
    private $_callbacks = [
        'themes'  => [],
        'plugins' => [],
    ];

    private function _addThemeCallback($alias, $priority, $hook, $callbackKey)
    {
        $this->_callbacks['themes'][$alias][$hook][$callbackKey] = $priority;
    }

    private function _addPluginCallback($alias, $priority, $hook, $callbackKey)
    {
        $this->_callbacks['plugins'][$alias][$hook][$callbackKey] = $priority;
    }

    public function getCallbacks()
    {
        return $this->_callbacks;
    }

    public function monitor()
    {
        global $wp_filter;

        $hook = current_filter();
        if (!isset($wp_filter[$hook])) {
            return;
        }

        $this->_monitorAction($wp_filter[$hook], $hook);
    }

    /**
     * @param WP_Hook $action
     * @param $hook
     */
    private function _monitorAction(WP_Hook $action, $hook)
    {
        foreach ($action->callbacks as $priority => $callbacks) {
            foreach ($callbacks as $callbackKey => $callback) {
                try {
                    try {
                        if (is_array($callback['function'])) {
                            if (is_object($callback['function'][0])) {
                                $class = get_class($callback['function'][0]);
                                $ref = new ReflectionMethod($class, $callback['function'][1]);
                            } elseif (is_string($callback['function'][0])) {
                                $ref = new ReflectionMethod($callback['function'][0], $callback['function'][1]);
                            } else {
                                return;
                            }
                        } elseif ($callback['function'] instanceof Closure) {
                            $ref = new ReflectionFunction($callback['function']);
                        } elseif (is_string($callback['function'])) {
                            $ref = isset($callback['name']) ? new ReflectionMethod($callback['name']) : new ReflectionMethod($callback['function']);
                        } else {
                            return;
                        }
                    } catch (\ReflectionException $e) {
                        if(is_string($callback['function'])) {
                            $ref = new ReflectionFunction($callback['function']);
                        } else {
                            throw new ReflectionException($e->getMessage(), $e->getCode(), $e);
                        }
                    }

                    $filename = $ref->getFileName();

                    if ($this->_isThemePath($filename)) {
                        $this->_addThemeCallback($this->_getVendorAliasFromPath($filename), $priority, $hook, $callbackKey);
                    } elseif ($this->_isPluginPath($filename)) {
                        $this->_addPluginCallback($this->_getVendorAliasFromPath($filename), $priority, $hook, $callbackKey);
                    }

                } catch (\ReflectionException $e) {
                    //$callback['error'] = new \WP_Error( 'reflection_exception', $e->getMessage() );
                }
            }
        }
    }

    private function _isThemePath($path)
    {
        return (bool)preg_match('/\/themes\//', $path);
    }

    private function _isPluginPath($path)
    {
        return (bool)preg_match('/\/plugins\//', $path);
    }

    private function _getVendorAliasFromPath($path)
    {
        preg_match('/\/(themes|plugins|cache)\/(.*?)\//', $path, $matches);

        return wpm_array_get($matches, 2);
    }

    public function removeThemeHooks($alias)
    {
        foreach (wpm_array_get($this->_callbacks, "themes.{$alias}", []) as $hook => $callbacks) {
            foreach ($callbacks as $callbackKey => $priority) {
                remove_filter($hook, $callbackKey, $priority);
            }
        }
    }

    public function removePluginHooks($alias)
    {
        foreach (wpm_array_get($this->_callbacks, "plugins.{$alias}", []) as $hook => $callbacks) {
            foreach ($callbacks as $callbackKey => $priority) {
                if (strpos($hook, 'nonce') === false) {
                    remove_filter($hook, $callbackKey, $priority);
                }
            }
        }
    }
}
