<?php

class MBLConnectorPublic
{
    public function __construct()
    {
        add_action('wp', [$this, 'setHooks']);
    }

    public function setHooks()
    {
        if (!is_admin() && mbl_is_mbl_page()) {
            $this->_updateSettings();
            $this->_addStyles();
            $this->_addScripts();

            add_action('wp_enqueue_scripts', [$this, 'filterCSS'], MBLConnectorCore::PRIORITY_MAX);
            add_action('wp_enqueue_scripts', [$this, 'filterJS'], MBLConnectorCore::PRIORITY_MAX);
            add_action('wp_enqueue_scripts', [$this, 'filterHooks'], MBLConnectorCore::PRIORITY_MIN);
            add_action('wp_enqueue_scripts', [$this, 'filterHooks'], MBLConnectorCore::PRIORITY_MAX);

            add_action('wp_head', [$this, 'filterHooks'], MBLConnectorCore::PRIORITY_MIN);
            add_action('wp_head', [$this, 'filterHooks'], MBLConnectorCore::PRIORITY_MAX);
            add_action('after_setup_theme', [$this, 'filterHooks'], MBLConnectorCore::PRIORITY_MAX);

            add_action('wpm_footer', [$this, 'addFooterCode']);
            add_action('wpm_head', [$this, 'addHeaderCode']);

            add_filter('mblp_wp_footer', '__return_false');
            add_filter('mblp_wp_footer', '__return_false');
            add_filter('mblp_add_wp_script', '__return_false');
            add_filter('mblp_add_wp_style', '__return_false');

            add_action('after_setup_theme', [$this, 'removeAdminBar']);
            add_filter('show_admin_bar', '__return_false');

            add_action('wp_footer', [$this, 'filterCSS'], MBLConnectorCore::PRIORITY_MIN);
            add_action('wp_footer', [$this, 'filterJS'], MBLConnectorCore::PRIORITY_MIN);
            add_action('wp_footer', [$this, 'filterHooks'], MBLConnectorCore::PRIORITY_MIN);

            add_action('wp_print_footer_scripts', [$this, 'filterCSS'], MBLConnectorCore::PRIORITY_MAX);
            add_action('wp_print_footer_scripts', [$this, 'filterJS'], MBLConnectorCore::PRIORITY_MAX);
            add_action('wp_print_footer_scripts', [$this, 'filterHooks'], MBLConnectorCore::PRIORITY_MIN);
        }
    }

    public function filterHooks()
    {
        foreach ($this->_getThemeSettings() as $theme => $options) {
            if (wpm_array_get($options, 'allow', 'off') == 'off') {
                MBLConnectorMonitorRunner::removeThemeHooks($theme);
            }
        }

        foreach ($this->_getPluginsSettings() as $alias => $options) {
            if (wpm_array_get($options, 'allow', 'on') == 'off') {
                MBLConnectorMonitorRunner::removePluginHooks($alias);
            }
        }
    }

    private function _getThemeSettings()
    {
        $defaultSettings = [];
        $theme = wp_get_theme();

        foreach (array_unique([$theme->get_stylesheet(), $theme->get_template()]) as $theme) {
            $defaultSettings[$theme] = [
                'allow' => 'off',
            ];
        }

        return wpm_get_option('mbl_connector.themes', $defaultSettings);
    }

    private function _getPluginsSettings()
    {
        if (!function_exists('get_plugins')) {
            require_once ABSPATH . 'wp-admin/includes/plugin.php';
        }

        $defaultSettings = [];

        $allPlugins = get_plugins();
        $plugins = [];

        foreach ($allPlugins as $path => $plugin) {
            if (!is_plugin_active($path)) {
                continue;
            }

            $alias = wpm_array_get(explode('.', wpm_array_get(explode('/', $path), 0, '')), 0);

            if (MBLConnectorCore::isDefaultPluginAsset($plugin['Name']) || MBLConnectorCore::isDefaultPluginAsset($alias)) {
                continue;
            }

            $plugins[] = $alias;
        }

        foreach ($plugins as $alias) {
            $defaultSettings[$alias] = [
                'allow' => 'off',
            ];
        }

        return array_merge($defaultSettings, wpm_get_option('mbl_connector.plugins', $defaultSettings));
    }

    private function _updateSettings()
    {
        $themePathArray = explode('/', get_template_directory_uri());
        $themeAlias = end($themePathArray);

        MBLConnectorCore::setOption("mbl_connector.plugins.{$themeAlias}", [
            'allow' => 'off',
        ]);
    }

    private function _addStyles()
    {
        add_action("wpm_head", [$this, 'addStyles']);
    }


    private function _addScripts()
    {
        add_action("wpm_footer", [$this, 'addScripts']);
    }


    public function addStyles()
    {
    }

    public function addScripts()
    {
    }

    public function filterCSS()
    {
        global $wp_styles;

        foreach ($wp_styles->queue as $asset) {
            if (!$this->_isCSSAllowed($asset)) {
                wp_dequeue_style($asset);
            }
        }
    }

    private function _isCSSAllowed($handle)
    {
        global $wp_styles;

        $alias = $this->_getVendorAliasFromPath($wp_styles->registered[$handle]->src);

        $allowed = wpm_option_is("mbl_connector.plugins.{$alias}.allow", 'on', 'on')
                   || wpm_option_is("mbl_connector.themes.{$alias}.allow", 'on', 'off')
                   || MBLConnectorCore::isDefaultPluginAsset($alias);

        if ($alias == MBLConnectorCore::THEME_SLUG) {
            $allowed = false;
        }

        return apply_filters('mbl_connector_public_css', $allowed, $handle);
    }

    private function _getVendorAliasFromPath($path)
    {
        preg_match('/\/(themes|plugins|cache)\/(.*?)\//', $path, $matches);

        return wpm_array_get($matches, 2);
    }

    public function filterJS()
    {
        global $wp_scripts;

        foreach ($wp_scripts->queue as $handle) {
            if (!$this->_isJSAllowed($handle)) {
                wp_dequeue_script($handle);
            }
        }
    }

    private function _isJSAllowed($handle)
    {
        global $wp_scripts;

        $alias = $this->_getVendorAliasFromPath($wp_scripts->registered[$handle]->src);

        $allowed = wpm_option_is("mbl_connector.plugins.{$alias}.allow", 'on', 'on')
                   || wpm_option_is("mbl_connector.themes.{$alias}.allow", 'on', 'off')
                   || MBLConnectorCore::isDefaultPluginAsset($alias);

        if ($alias == MBLConnectorCore::THEME_SLUG) {
            $allowed = false;
        }

        $allowed = $allowed && !MBLConnectorCore::isDisabledAsset($handle) && !MBLConnectorCore::isDisabledAsset($alias);

        return apply_filters('mbl_connector_public_js', $allowed, $handle);
    }

    public function addFooterCode()
    {
        wp_footer();
    }

    public function addHeaderCode()
    {
        wp_head();
    }

    public function removeAdminBar()
    {
        show_admin_bar(false);
    }
}